@@ -563,14 +563,19 @@ void _dl_boot(int args){
 	    else
 	    {
 	      _dl_fdprintf(2, "%s: can't load library '%s'\n",
-			   _dl_progname,str);
+			   _dl_progname, str);
 	      _dl_exit(15);
 	    }
 	  }
 	  else
 	  {
 	    if (_dl_trace_loaded_objects && !tpnt1->usage_count)
-	      _dl_fdprintf(1, "\t%s => %s\n", str, tpnt1->libname);
+	    {
+	      /* this is a real hack to make ldd not print the
+		 library itself when run on a library. */
+	      if (_dl_strcmp(_dl_progname, str) != 0)
+		_dl_fdprintf(1, "\t%s => %s\n", str, tpnt1->libname);
+	    }
 	    rpnt->next = 
 	      (struct dyn_elf *) _dl_malloc(sizeof(struct dyn_elf));
 	    _dl_memset (rpnt->next, 0, sizeof (*(rpnt->next)));
