@@ -138,10 +138,16 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
 	char * full_libname){
   char * pnt, *pnt1, *pnt2;
   struct elf_resolve *tpnt1 = NULL;
-  char mylibname[1024];
+  char mylibname[2050];
   char * libname;
 
   _dl_internal_error_number = 0;
+
+  /* quick hack to ensure mylibname buffer doesn't overflow.  don't
+     allow full_libname or any directory to be longer than 1024. */
+  if (_dl_strlen(full_libname) > 1024)
+    goto goof;
+
   pnt = libname = full_libname;
   while (*pnt) {
     if(*pnt == '/') libname = pnt+1;
@@ -185,7 +191,14 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
 	pnt1 += (unsigned int) tpnt->loadaddr + tpnt->dynamic_info[DT_STRTAB];
 	while(*pnt1){
 	  pnt2 = mylibname;
-	  while(*pnt1 && *pnt1 != ':') *pnt2++ = *pnt1++;
+          while(*pnt1 && *pnt1 != ':') {
+            if (pnt2 - mylibname < 1024)
+              *pnt2++ = *pnt1++;
+            else
+              pnt1++;
+          }
+          if (pnt2 - mylibname >= 1024)
+            break;
 	  if(pnt2[-1] != '/') *pnt2++ = '/';
 	  pnt = libname;
 	  while(*pnt) *pnt2++  = *pnt++;
@@ -204,7 +217,14 @@ struct elf_resolve * _dl_load_shared_library(struct elf_resolve * tpnt,
   if (pnt1 && *pnt1) {
     while (*pnt1) {
       pnt2 = mylibname;
-      while(*pnt1 && *pnt1 != ':' && *pnt1 != ';') *pnt2++ = *pnt1++;
+      while(*pnt1 && *pnt1 != ':' && *pnt1 != ';') {
+        if (pnt2 - mylibname < 1024)
+          *pnt2++ = *pnt1++;
+       else
+         pnt1++;
+      }
+      if (pnt2 - mylibname >= 1024)
+        break;
       if(pnt2[-1] != '/') *pnt2++ = '/';
       pnt = libname;
       while(*pnt) *pnt2++  = *pnt++;
