@@ -252,20 +252,18 @@ int _dl_parse_relocation_information(struct elf_resolve * tpnt, int rel_addr,
       *reloc_addr += (unsigned int) tpnt->loadaddr + rpnt->r_addend;
       break;
     case R_SPARC_HI22:
-      if (symbol_addr)
-        *reloc_addr = (*reloc_addr & 0xffc00000)|(symbol_addr >> 10);
-      else {
+      if (!symbol_addr)
         symbol_addr = tpnt->loadaddr + rpnt->r_addend;
-        *reloc_addr = (*reloc_addr & 0xffc00000)|(symbol_addr >> 10);
-      }
+      else
+	symbol_addr += rpnt->r_addend;
+      *reloc_addr = (*reloc_addr & 0xffc00000)|(symbol_addr >> 10);
       break;
     case R_SPARC_LO10:
-      if (symbol_addr)
-        *reloc_addr = (*reloc_addr & ~0x3ff)|(symbol_addr & 0x3ff);
-      else {
+      if (!symbol_addr)
         symbol_addr = tpnt->loadaddr + rpnt->r_addend;
-        *reloc_addr = (*reloc_addr & ~0x3ff)|(symbol_addr & 0x3ff);	    
-      }
+      else
+	symbol_addr += rpnt->r_addend;
+      *reloc_addr = (*reloc_addr & ~0x3ff)|(symbol_addr & 0x3ff);
       break;
     case R_SPARC_WDISP30:
       *reloc_addr = (*reloc_addr & 0xc0000000)|
