@@ -164,7 +164,7 @@ shared_loader(int func, ...)
 #ifndef DEBUG_LIB
     unsigned *mapinfo = NULL;
 #endif
-    char *libs[10];
+    char *libs[32];
     int nlibs=0;
     int nowarn;
     int keepdir;
@@ -322,6 +322,11 @@ shared_loader(int func, ...)
 		.text section. This is passed to ldpreload() below */
 	    if (preload || callback)
 	    {
+	        if (nlibs >= (sizeof libs / sizeof libs[0]))
+		{
+		    fdprintf(2, "%s: too many preloads\n",argv0);
+		    exit(EXIT_FATAL);
+		}
 	        libs[nlibs] = alloca(strlen(buffer)+1);
 	        strcpy(libs[nlibs], buffer);
 	        nlibs++;
