@@ -65,8 +65,7 @@ int nolinks = 0;		/* don't update links */
 char *conffile = LDSO_CONF;	/* default cache file */
 char *cachefile = LDSO_CACHE;	/* default cache file */
 void cache_print(void);
-void cache_dolib(char *dir, char *so, char *lib, int);
-void cache_rmlib(char *dir, char *so);
+void cache_dolib(char *dir, char *so, int libtype);
 void cache_write(void);
 
 char *readsoname(FILE *file);
@@ -120,7 +119,7 @@ char *xstrdup(char *str)
 
 /* if shared library, return a malloced copy of the soname and set the
    type, else return NULL */
-char *is_shlib(char *dir, char *name, int *type)
+char *is_shlib(char *dir, char *name, int *type, int *islink)
 {
     char *good = NULL;
     char *cp, *cp2;
@@ -147,21 +146,20 @@ char *is_shlib(char *dir, char *name, int *type)
 	/* first, make sure it's a regular file */
 	if (lstat(buff, &statbuf))
 	    warn("can't lstat %s (%s), skipping", buff, strerror(errno));
-	else if (!S_ISREG(statbuf.st_mode))
-	{
-	    /* silently ignore symlinks */
-	    if (!S_ISLNK(statbuf.st_mode))
-	        warn("%s is not a regular file, skipping", 
-		     buff, strerror(errno));
-	}
+	else if (!S_ISREG(statbuf.st_mode) && !S_ISLNK(statbuf.st_mode))
+	    warn("%s is not a regular file or symlink, skipping", 
+		 buff, strerror(errno));
 	else
 	{
+	    /* is it a regular file or a symlink */
+	    *islink = S_ISLNK(statbuf.st_mode);
+
 	    /* then try opening it */
 	    if (!(file = fopen(buff, "rb")))
 		warn("can't open %s (%s), skipping", buff, strerror(errno));
 	    else
 	    {
-		/* now make sure it's a DLL or ELF library */
+		/* now make sure it's a shared library */
 		if (fread(&exec, sizeof exec, 1, file) < 1)
 		    warn("can't read header from %s, skipping", buff);
 		else if (N_MAGIC(exec) != ZMAGIC && N_MAGIC(exec) != QMAGIC)
@@ -169,34 +167,38 @@ char *is_shlib(char *dir, char *name, int *type)
 		    elf_hdr = (struct elfhdr *) &exec;
 		    if (elf_hdr->e_ident[0] != 0x7f ||
 			strncmp(&elf_hdr->e_ident[1], "ELF",3) != 0)
-			warn("%s is not a DLL or ELF library, skipping", buff);
+		    {
+		        /* silently ignore linker scripts */
+		        if (strncmp((char *)&exec, "/* GNU ld", 9) != 0)
+			    warn("%s is not a shared library, skipping", buff);
+		    }
 		    else
 		    {
-			if ((good = readsoname(file)) == NULL)
+			if (*islink || (good = readsoname(file)) == NULL)
 			    good = xstrdup(name);
 			else
 			{
 			    /* if the soname does not match the filename,
-			       we assume it is for compile-time compatibility
-			       and ignore it. */
+			       issue a warning, but only in debug mode. */
 			    int len = strlen(good);
-			    if (strncmp(good, name, len) != 0 ||
-				(name[len] != '\0' && name[len] != '.'))
-			    {
-			        warn("%s has inconsistent soname (%s), "
-				     "skipping", buff, good);
-			        free(good);
-				good = NULL;
-			    }
+			    if (debug && (strncmp(good, name, len) != 0 ||
+				(name[len] != '\0' && name[len] != '.')))
+			        warn("%s has inconsistent soname (%s)",
+				     buff, good);
 			}
 			*type = LIB_ELF;
 		    }
 		}
 		else
 		{
-		    good = xmalloc(cp - name + 1);
-		    strncpy(good, name, cp - name);
-		    good[cp - name] = '\0';
+		    if (*islink)
+		        good = xstrdup(name);
+		    else
+		    {
+		        good = xmalloc(cp - name + 1);
+			strncpy(good, name, cp - name);
+			good[cp - name] = '\0';
+		    }
 		    *type = LIB_DLL;
 		}
 		fclose(file);
@@ -278,6 +280,8 @@ struct lib
 {
     char *so;			/* soname of a library */
     char *name;			/* name of a library */
+    int libtype;		/* type of a library */
+    int islink;			/* is it a symlink */
     struct lib *next;		/* next library in list */
 };
 
@@ -288,7 +292,7 @@ void scan_dir(char *name)
     struct dirent *ent;
     char *so;
     struct lib *lp, *libs = NULL;
-    int libtype;
+    int libtype, islink;
 
     /* let 'em know what's going on */
     if (verbose)
@@ -305,23 +309,24 @@ void scan_dir(char *name)
     while ((ent = readdir(dir)) != NULL)
     {
 	/* if it's not a shared library, don't bother */
-	if ((so = is_shlib(name, ent->d_name, &libtype)) == NULL)
+	if ((so = is_shlib(name, ent->d_name, &libtype, &islink)) == NULL)
 	    continue;
 
-	if (!nocache)
-	    cache_dolib(name, so, ent->d_name, libtype);
-
 	/* have we already seen one with the same so name? */
 	for (lp = libs; lp; lp = lp->next)
 	{
 	    if (strcmp(so, lp->so) == 0)
 	    {
 	        /* we have, which one do we want to use? */
-	        if (libcmp(ent->d_name, lp->name) > 0)
+	        if ((!islink && lp->islink) ||
+		    (islink == lp->islink && 
+		     libcmp(ent->d_name, lp->name) > 0))
 		{
 		    /* let's use the new one */
 		    free(lp->name);
 		    lp->name = xstrdup(ent->d_name);
+		    lp->libtype = libtype;
+		    lp->islink = islink;
 		} 
 		break;
 	    }
@@ -333,6 +338,8 @@ void scan_dir(char *name)
 	    lp = xmalloc(sizeof *lp);
 	    lp->so = xstrdup(so);
 	    lp->name = xstrdup(ent->d_name);
+	    lp->libtype = libtype;
+	    lp->islink = islink;
 	    lp->next = libs;
 	    libs = lp;
 	}
@@ -346,9 +353,10 @@ void scan_dir(char *name)
     /* now we have all the latest libs, update the links */
     for (lp = libs; lp; lp = lp->next)
     {
-	link_shlib(name, lp->name, lp->so);
-	if (strcmp(lp->name, lp->so) != 0)
-	    cache_rmlib(name, lp->so);
+        if (!lp->islink)
+	    link_shlib(name, lp->name, lp->so);
+	if (!nocache)
+	    cache_dolib(name, lp->so, lp->libtype);
     }
 
     /* always try to clean up after ourselves */
@@ -401,7 +409,7 @@ int main(int argc, char **argv)
     int printcache = 0;
     char *cp, *dir, *so;
     char *extpath;
-    int libtype;
+    int libtype, islink;
 
     prog = argv[0];
     opterr = 0;
@@ -484,7 +492,7 @@ int main(int argc, char **argv)
 	    }
 
 	    /* we'd better do a little bit of checking */
-	    if ((so = is_shlib(dir, cp, &libtype)) == NULL)
+	    if ((so = is_shlib(dir, cp, &libtype, &islink)) == NULL)
 		error("%s%s%s is not a shared library", dir,
 		      (*dir && strcmp(dir, "/")) ? "/" : "", cp);
 
@@ -538,7 +546,6 @@ typedef struct liblist
     int liboffset;
     char *soname;
     char *libname;
-    int skip;
     struct liblist *next;
 } liblist_t;
 
@@ -558,18 +565,17 @@ static int liblistcomp(liblist_t *x, liblist_t *y)
     return res;
 }
 
-void cache_dolib(char *dir, char *so, char *lib, int libtype)
+void cache_dolib(char *dir, char *so, int libtype)
 {
     char fullpath[PATH_MAX];
     liblist_t *new_lib, *cur_lib;
 
     magic.nlibs++;
-    sprintf(fullpath, "%s/%s", dir, lib);
+    sprintf(fullpath, "%s/%s", dir, so);
     new_lib = xmalloc(sizeof (liblist_t));
     new_lib->flags = libtype;
     new_lib->soname = xstrdup(so);
     new_lib->libname = xstrdup(fullpath);
-    new_lib->skip = 0;
 
     if (lib_head == NULL || liblistcomp(new_lib, lib_head) > 0)
     {
@@ -587,25 +593,6 @@ void cache_dolib(char *dir, char *so, char *lib, int libtype)
     }
 }
 
-void cache_rmlib(char *dir, char *so)
-{
-    char fullpath[PATH_MAX];
-    liblist_t *cur_lib;
-
-    sprintf(fullpath, "%s/%s", dir, so);
-
-    for (cur_lib = lib_head; cur_lib != NULL; cur_lib = cur_lib->next)
-    {
-        if (strcmp(cur_lib->soname, so) == 0 &&
-	    strcmp(cur_lib->libname, fullpath) == 0 &&
-	    !cur_lib->skip)
-	{
-	    magic.nlibs--;
-	    cur_lib->skip = 1;
-	}
-    }
-}
-
 void cache_write(void)
 {
     int cachefd;
@@ -629,29 +616,23 @@ void cache_write(void)
 
     for (cur_lib = lib_head; cur_lib != NULL; cur_lib = cur_lib->next)
     {
-        if (!cur_lib->skip)
-	{
-	    cur_lib->sooffset = stroffset;
-	    stroffset += strlen(cur_lib->soname) + 1;
-	    cur_lib->liboffset = stroffset;
-	    stroffset += strlen(cur_lib->libname) + 1;
-	    if (write(cachefd, cur_lib, sizeof (libentry_t)) !=
-		sizeof (libentry_t))
-	        error("can't write %s (%s)", tempfile, strerror(errno));
-	}
+        cur_lib->sooffset = stroffset;
+	stroffset += strlen(cur_lib->soname) + 1;
+	cur_lib->liboffset = stroffset;
+	stroffset += strlen(cur_lib->libname) + 1;
+	if (write(cachefd, cur_lib, sizeof (libentry_t)) !=
+	    sizeof (libentry_t))
+	    error("can't write %s (%s)", tempfile, strerror(errno));
     }
 
     for (cur_lib = lib_head; cur_lib != NULL; cur_lib = cur_lib->next)
     {
-        if (!cur_lib->skip)
-	{
-	    if (write(cachefd, cur_lib->soname, strlen(cur_lib->soname) + 1)
-		!= strlen(cur_lib->soname) + 1)
-	        error("can't write %s (%s)", tempfile, strerror(errno));
-	    if (write(cachefd, cur_lib->libname, strlen(cur_lib->libname) + 1)
-		!= strlen(cur_lib->libname) + 1)
-	        error("can't write %s (%s)", tempfile, strerror(errno));
-	}
+      if (write(cachefd, cur_lib->soname, strlen(cur_lib->soname) + 1)
+	  != strlen(cur_lib->soname) + 1)
+	  error("can't write %s (%s)", tempfile, strerror(errno));
+      if (write(cachefd, cur_lib->libname, strlen(cur_lib->libname) + 1)
+	  != strlen(cur_lib->libname) + 1)
+	  error("can't write %s (%s)", tempfile, strerror(errno));
     }
 
     if (close(cachefd))
